package rmi_rabbit_herd;

/**
 * Title:        Remote rabbits
 * Description:  A test of remote rabbits herd
 * Copyright:    Copyright (c) 2001
 * Company:      PWr
 * @author SLK
 * @version 1.0
 */

import java.rmi.server.UnicastRemoteObject;
import java.rmi.server.Unreferenced;
import java.rmi.RemoteException;
import java.rmi.Naming;

public class RMI_animal extends UnicastRemoteObject implements animal, Unreferenced
{
   /** In our model of animal we use the following atributes only
    */

   // private section
   private boolean safe;
   private boolean pregnant;

   private int    condition;
   private String state;

   private void reset()
   {
      safe = true;
      pregnant = false;

      condition = 15;
      state = "Alive";
   }

   // public (interface) section
   public RMI_animal(String name) throws RemoteException
   {
      // registration part
      super();
      try
      {
         Naming.rebind(name, this);
      }
      catch (Exception e)
      {
         if (e instanceof RemoteException)
         {
            throw (RemoteException)e;
         }
         else
         {
            throw new RemoteException(e.getMessage());
         }
      }

      // animal part
      reset();
   }

   public boolean alive() throws RemoteException
   {
      return condition > 0;
   }

   public void be() throws RemoteException
   {
      if (condition > 0 && safe)
      {
         eat();

         if (pregnant)
         {
            bear();
         }
      }
      else
      {
         run();
      }
      safe = Math.random() < 0.2;
   }

   public void run() throws RemoteException
   {
      --condition;
      state = "Escaping " + condition;
   }
   public void eat()  throws RemoteException
   {
      if(condition < 40 && condition > 20)
      {
         condition += 5;
         pregnant = true;

         state = "Pregnant! " + condition;
      }
      else if(condition < 20)
      {
         condition += 10;

         state = "Almost fed " + condition;
      }
   }
   synchronized public void bear() throws RemoteException
   {
      pregnant = false;
      condition -= 20;
      state = "New rabbit born!";
   }

   public String status() throws RemoteException
   {
      return state;
   }

   public void unreferenced() // When will it work?!?
   {
      reset();
   }
}